"""
This module implement a Watermark filter.
"""
import os
from flask import current_app

from .interface import ImagineFilterInterface
from PIL import Image, ImageEnhance


class WatermarkFilter(ImagineFilterInterface):
    """
    Watermark filter
    """
    positions = ['top_left', 'top', 'top_right', 'left', 'center', 'right', 'bottom_left', 'bottom', 'bottom_right']

    image_path = None
    image = None
    size = None
    position = None
    opacity = None

    def __init__(self, **kwargs):
        """
        :param kwargs: dict
        """
        if 'image' in kwargs:
            self.image_path = os.path.normpath(str(kwargs['image']))
        else:
            raise ValueError('Watermark image path doesn\'t set.')

        if 'size' in kwargs:
            try:
                self.size = float(kwargs.get('size'))
                assert 0 <= self.size <= 1
            except Exception, e:
                raise ValueError('Unsupported size format: %s' % unicode(e))
        else:
            raise ValueError('Watermark size doesn\'t set.')

        if 'position' in kwargs:
            if kwargs['position'] in self.positions:
                self.position = kwargs.get('position')
            else:
                raise ValueError('Unsupported watermark position: %s' % kwargs.get('position'))
        else:
            raise ValueError('Watermark position doesn\'t set.')

        if 'opacity' in kwargs:
            try:
                self.opacity = float(kwargs.get('opacity', 0.3))
                assert 0 <= self.opacity <= 1
            except Exception, e:
                raise ValueError('Unsupported opacity format: %s' % unicode(e))
        else:
            self.opacity = 0.3

    def apply(self, resource):
        """
        Apply filter to resource
        :param resource: Image.Image
        :return: Image.Image
        """
        if not isinstance(resource, Image.Image):
            raise ValueError('Unknown resource format')

        if resource.mode != 'RGBA':  # pragma: no cover
            resource = resource.convert('RGBA')

        layer = Image.new('RGBA', resource.size, (0, 0, 0, 0))
        image, left, upper = getattr(self, '_' + self.position + '_position')(resource)
        layer.paste(image, (left, upper))

        return Image.composite(layer, resource, layer)

    def _top_left_position(self, resource):
        """
        Place watermark to top left position
        :param layer: Image.Image
        :param resource: Image.Image
        :return: Image.Image
        """
        image = self._get_scaled_image(resource)

        return image, 0, 0

    def _top_position(self, resource):
        """
        Place watermark to top position
        :param layer: Image.Image
        :param resource: Image.Image
        :return: Image.Image
        """
        image = self._get_scaled_image(resource)

        left = int(round(resource.size[0] / 2 - image.size[0] / 2))
        upper = 0

        return image, left, upper

    def _top_right_position(self, resource):
        """
        Place watermark to top right position
        :param layer: Image.Image
        :param resource: Image.Image
        :return: Image.Image
        """
        image = self._get_scaled_image(resource)

        left = int(round(resource.size[0] - image.size[0]))
        upper = 0

        return image, left, upper

    def _left_position(self, resource):
        """
        Place watermark to left position
        :param layer: Image.Image
        :param resource: Image.Image
        :return: Image.Image
        """
        image = self._get_scaled_image(resource)

        left = 0
        upper = int(round(resource.size[1] / 2 - image.size[1] / 2))

        return image, left, upper

    def _center_position(self, resource):
        """
        Place watermark to center position
        :param layer: Image.Image
        :param resource: Image.Image
        :return: Image.Image
        """
        image = self._get_scaled_image(resource)

        left = int(round(resource.size[0] / 2 - image.size[0] / 2))
        upper = int(round(resource.size[1] / 2 - image.size[1] / 2))

        return image, left, upper

    def _right_position(self, resource):
        """
        Place watermark to right position
        :param layer: Image.Image
        :param resource: Image.Image
        :return: Image.Image
        """
        image = self._get_scaled_image(resource)

        left = int(round(resource.size[0] - image.size[0]))
        upper = int(round(resource.size[1] / 2 - image.size[1] / 2))

        return image, left, upper

    def _bottom_left_position(self, resource):
        """
        Place watermark to bottom left position
        :param layer: Image.Image
        :param resource: Image.Image
        :return: Image.Image
        """
        image = self._get_scaled_image(resource)

        left = 0
        upper = int(round(resource.size[1] - image.size[1]))

        return image, left, upper

    def _bottom_position(self, resource):
        """
        Place watermark to bottom position
        :param layer: Image.Image
        :param resource: Image.Image
        :return: Image.Image
        """
        image = self._get_scaled_image(resource)

        left = int(round(resource.size[0] / 2 - image.size[0] / 2))
        upper = int(round(resource.size[1] - image.size[1]))

        return image, left, upper

    def _bottom_right_position(self, resource):
        """
        Place watermark to bottom right position
        :param layer: Image.Image
        :param resource: Image.Image
        :return: Image.Image
        """
        image = self._get_scaled_image(resource)

        left = int(round(resource.size[0] - image.size[0]))
        upper = int(round(resource.size[1] - image.size[1]))

        return image, left, upper

    def _get_image(self):
        """
        Prepare watermark image
        :return: Image.Image
        """
        if self.image is None:
            image_path = '%s/%s' % (current_app.static_folder, os.path.normpath(self.image_path))
            try:
                self.image = Image.open(image_path)
                self._reduce_opacity()
            except Exception, e:
                raise ValueError('Unsupported watermark format: %s' % unicode(e))

        return self.image

    def _reduce_opacity(self):
        """
        Reduce opacity for watermark image.
        """
        if self.image.mode != 'RGBA':
            image = self.image.convert('RGBA')
        else:
            image = self.image.copy()

        alpha = image.split()[3]
        alpha = ImageEnhance.Brightness(alpha).enhance(self.opacity)
        image.putalpha(alpha)

        self.image = image

    def _get_scaled_image(self, resource):
        """
        Get scaled watermark image
        :param resource: Image.Image
        :return: Image.Image
        """
        image = self._get_image()
        original_width, original_height = resource.size

        k = image.size[0] / float(image.size[1])

        if image.size[0] >= image.size[1]:
            target_width = int(original_width * self.size)
            target_height = int(target_width / k)
        else:
            target_height = int(original_height * self.size)
            target_width = int(target_height * k)

        image = image.resize((target_width, target_height), Image.ANTIALIAS)

        return image




"""
This module implement a storage adapter interface.
"""


class ImagineAdapterInterface(object):
    """
    Storage adapter interface
    """
    def get_item(self, path):
        """
        Get resource item
        :param path: string
        :return: Image
        """
        raise NotImplementedError()

    def create_cached_item(self, path, content):
        """
        Create cached resource item
        :param path: string
        :param content: Image
        :return:
        """
        raise NotImplementedError()

    def get_cached_item(self, path):
        """
        Get cached resource item
        :param path: string
        :return:
        """
        raise NotImplementedError()

    def check_cached_item(self, path):
        """
        Check for cached resource item exists
        :param path: string
        :return:
        """
        raise NotImplementedError()

    def remove_cached_item(self, path):
        """
        Remove cached resource item
        :param path: string
        :return:
        """
        raise NotImplementedError()



"""
This module implement a Upscale filter.
"""
from .interface import ImagineFilterInterface
from PIL import Image


class UpscaleFilter(ImagineFilterInterface):
    """
    Upscale filter
    """
    size = None

    def __init__(self, **kwargs):
        """
        :param kwargs: dict
        """
        if 'min' in kwargs and isinstance(kwargs['min'], list) and len(kwargs['min']) == 2:
            self.size = kwargs.get('min')
        else:
            raise ValueError('Unsupported configuration')

    def apply(self, resource):
        """
        Apply filter to resource.
        :param resource: Image
        :return: Image
        """
        if not isinstance(resource, Image.Image):
            raise ValueError('Unsupported resource format: %s' % str(type(resource)))

        original_width, original_height = resource.size

        if original_width < self.size[0] and original_height < self.size[1]:
            k = original_width / float(original_height)

            if original_width >= original_height:
                target_width = self.size[0]
                target_height = int(target_width / k)
            else:
                target_height = self.size[1]
                target_width = int(target_height * k)

            resource = resource.resize((target_width, target_height), Image.ANTIALIAS)

        return resource



"""
Flask Imagine extension.
"""
import logging

from flask import current_app, abort, redirect

from .adapters import ImagineFilesystemAdapter
from .filters import *
from .helpers.regex_route import RegexConverter

LOGGER = logging.getLogger(__file__)


class Imagine(object):
    """
    Flask Imagine extension
    """
    adapters = {
        'fs': ImagineFilesystemAdapter
    }
    filters = {
        'autorotate': AutorotateFilter,
        'crop': CropFilter,
        'downscale': DownscaleFilter,
        'relative_resize': RelativeResizeFilter,
        'rotate': RotateFilter,
        'thumbnail': ThumbnailFilter,
        'upscale': UpscaleFilter,
        'watermark': WatermarkFilter
    }

    filter_sets = {}
    adapter = None

    def __init__(self, app=None):
        """
        :param app: Flask application
        """
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        """
        :param app: Flask application
        """
        if not hasattr(app, 'extensions'):  # pragma: no cover
            app.extensions = {}
        app.extensions['imagine'] = self

        self._set_defaults(app)

        if isinstance(app.config['IMAGINE_ADAPTERS'], dict):
            self.adapters.update(app.config['IMAGINE_ADAPTERS'])
        if isinstance(app.config['IMAGINE_FILTERS'], dict):
            self.filters.update(app.config['IMAGINE_FILTERS'])

        self._handle_adapter(app)
        self._handle_filter_sets(app)

        self._add_url_rule(app)

    @classmethod
    def _set_defaults(cls, app):
        """
        Set default configuration parameters
        :param app: Flask application
        :return: Flask application
        """
        app.config.setdefault('IMAGINE_URL', '/media/cache/resolve')
        app.config.setdefault('IMAGINE_NAME', 'imagine')
        app.config.setdefault('IMAGINE_CACHE_ENABLED', True)

        app.config.setdefault('IMAGINE_ADAPTERS', {})
        app.config.setdefault('IMAGINE_FILTERS', {})

        app.config.setdefault('IMAGINE_ADAPTER', {
            'name': 'fs',
            'source_folder': 'static',
            'cache_folder': 'cache'
        })

        app.config.setdefault('IMAGINE_FILTER_SETS', {})

        return app

    def _handle_adapter(self, app):
        """
        Handle storage adapter configuration
        :param app: Flask application
        """
        if 'IMAGINE_ADAPTER' in app.config \
                and 'name' in app.config['IMAGINE_ADAPTER'] \
                and app.config['IMAGINE_ADAPTER']['name'] in self.adapters.keys():
            self.adapter = self.adapters[app.config['IMAGINE_ADAPTER']['name']](
                **app.config['IMAGINE_ADAPTER']
            )
        else:
            raise ValueError('Unknown adapter: %s' % unicode(app.config['IMAGINE_ADAPTER']))

    def _handle_filter_sets(self, app):
        """
        Handle filter sets
        :param app: Flask application
        """
        if 'IMAGINE_FILTER_SETS' in app.config and isinstance(app.config['IMAGINE_FILTER_SETS'], dict):
            for filter_name, filters_settings in app.config['IMAGINE_FILTER_SETS'].iteritems():
                filter_set = []
                if isinstance(filters_settings, dict) and 'filters' in filters_settings:
                    for filter_type, filter_settings in filters_settings['filters'].iteritems():
                        if filter_type in self.filters:
                            filter_item = self.filters[filter_type](**filter_settings)
                            if isinstance(filter_item, ImagineFilterInterface):
                                filter_set.append(filter_item)
                            else:
                                raise ValueError('Filter must be implement ImagineFilterInterface')
                        else:
                            raise ValueError('Unknown filter type: %s' % filter_type)

                    filter_config = {'filters': filter_set}
                    if 'cached' in filters_settings:
                        filter_config['cached'] = filters_settings['cached']
                    else:
                        filter_config['cached'] = app.config['IMAGINE_CACHE_ENABLED']

                    self.filter_sets.update({filter_name: filter_config})
                else:
                    raise ValueError('Wrong settings for filter: %s' % filter_name)
        else:
            raise ValueError('Filters configuration does not present')

    def _add_url_rule(self, app):
        """
        Add url rule for get filtered images
        :param app: Flask application
        :return: Flask application
        """
        app.url_map.converters['regex'] = RegexConverter
        app.add_url_rule(
            app.config['IMAGINE_URL'] + '/<regex("[^\/]+"):filter_name>/<path:path>',
            app.config['IMAGINE_NAME'],
            self.handle_request
        )

        return app

    def handle_request(self, filter_name, path):
        """
        Handle image request
        :param filter_name: filter_name
        :param path: image_path
        :return:
        """
        if filter_name in self.filter_sets:
            if self.filter_sets[filter_name]['cached']:
                if self.adapter.check_cached_item('%s/%s' % (filter_name, path)):
                    return redirect(
                        '%s/%s/%s' % (
                            self.adapter.source_folder,
                            self.adapter.cache_folder,
                            '%s/%s' % (filter_name, path)
                        )
                    )

            resource = self.adapter.get_item(path)

            if resource:
                for filter_item in self.filter_sets[filter_name]['filters']:
                    resource = filter_item.apply(resource)

                return redirect(self.adapter.create_cached_item('%s/%s' % (filter_name, path), resource))
            else:
                LOGGER.warning('File "%s" not found.' % path)
                abort(404)
        else:
            LOGGER.warning('Filter "%s" not found.' % filter_name)
            abort(404)

    def clear_cache(self, path, filter_name=None):
        """
        Clear cache for resource path.
        :param path: str
        :param filter_name: str or None
        """
        if filter_name:
            self.adapter.remove_cached_item('%s/%s' % (filter_name, path))
        else:
            for filter_name in self.filter_sets.iterkeys():
                self.adapter.remove_cached_item('%s/%s' % (filter_name, path))


def imagine_cache_clear(path, filter_name=None):
    """
    Clear cache for resource path.
    :param path: str
    :param filter_name: str or None
    """
    self = current_app.extensions['imagine']
    self.clear_cache(path, filter_name)




"""
This module implement a Crop filter.
"""
from .interface import ImagineFilterInterface
from PIL import Image


class CropFilter(ImagineFilterInterface):
    """
    Crop filter
    """
    start = None
    size = None

    def __init__(self, **kwargs):
        """
        :param kwargs: dict
        """
        if 'start' in kwargs and isinstance(kwargs['start'], list) and len(kwargs['start']) == 2:
            self.start = kwargs.get('start')
        else:
            raise ValueError('Unknown start position.')

        if 'size' in kwargs and isinstance(kwargs['size'], list) and len(kwargs['size']) == 2:
            self.size = kwargs.get('size')
        else:
            raise ValueError('Unknown image size.')

    def apply(self, resource):
        """
        Apply filter to resource
        :param resource: Image
        :return: Image
        """
        if not isinstance(resource, Image.Image):
            raise ValueError('Unknown resource format')

        original_width, original_height = resource.size

        if self.start[0] < original_width or self.start[1] < original_height:
            left = self.start[0]
            upper = self.start[1]
        else:
            left = 0
            upper = 0

        right = self.start[0] + self.size[0]
        lower = self.start[1] + self.size[1]

        resource = resource.crop(
            (
                left,
                upper,
                right if right < original_width else original_width,
                lower if lower < original_height else original_height
            )
        )

        return resource



"""
This module implement a Relative resize filter.
"""
from .interface import ImagineFilterInterface
from PIL import Image


class RelativeResizeFilter(ImagineFilterInterface):
    """
    Relative resize filter
    """
    methods = ['heighten', 'widen', 'increase', 'decrease', 'scale']
    method = None
    value = None

    def __init__(self, **kwargs):
        """
        :param kwargs: dict
        """
        for method in self.methods:
            if method in kwargs:
                self.method = method

        if not self.method:
            raise ValueError('Unknown method')

        if self.method == 'scale':
            try:
                self.value = float(kwargs[self.method])
            except Exception, e:
                raise ValueError('Wrong value type: %s' % unicode(e))
        else:
            try:
                self.value = int(kwargs[self.method])
            except Exception, e:
                raise ValueError('Wrong value type: %s' % unicode(e))

    def apply(self, resource):
        """
        :param resource: Image
        :return: Image
        """
        if isinstance(resource, Image.Image):
            return getattr(self, '_' + self.method)(resource)
        else:
            raise ValueError('Unsupported resource format: %s' % str(type(resource)))

    def _heighten(self, resource):
        """
        Change image size by height
        :param resource: Image
        :return: Image
        """
        original_width, original_height = resource.size

        target_height = self.value
        target_width = int((float(target_height) / original_height) * original_width)

        resource = resource.resize((target_width, target_height), Image.ANTIALIAS)

        return resource

    def _widen(self, resource):
        """
        Change image size by width
        :param resource: Image
        :return: Image
        """
        original_width, original_height = resource.size

        target_width = self.value
        target_height = int((float(target_width) / original_width) * original_height)

        resource = resource.resize((target_width, target_height), Image.ANTIALIAS)

        return resource

    def _increase(self, resource):
        """
        Increase image size
        :param resource: Image
        :return: Image
        """
        original_width, original_height = resource.size

        target_width = original_width + self.value
        target_height = original_height + self.value

        resource = resource.resize((target_width, target_height), Image.ANTIALIAS)

        return resource

    def _decrease(self, resource):
        """
        Decrease image size
        :param resource:
        :return:
        """
        original_width, original_height = resource.size

        if original_width > self.value and original_height > self.value:
            target_width = original_width - self.value
            target_height = original_height - self.value

            resource = resource.resize((target_width, target_height), Image.ANTIALIAS)

            return resource
        else:
            raise ValueError('Image dimensions less than or equal to filter value.')

    def _scale(self, resource):
        """
        Scale image
        :param resource: Image
        :return: Image
        """
        original_width, original_height = resource.size

        target_width = int(round(original_width * self.value))
        target_height = int(round(original_height * self.value))

        resource = resource.resize((target_width, target_height), Image.ANTIALIAS)

        return resource



"""
This module implement a filter interface.
"""


class ImagineFilterInterface(object):
    """
    Filter interface
    """
    def apply(self, resource):
        """
        Apply filter to resource
        :param resource: Image
        :return: Image
        """
        raise NotImplementedError()



"""
This module implement a Thumbnail filter.
"""
from PIL import Image
from .interface import ImagineFilterInterface


class ThumbnailFilter(ImagineFilterInterface):
    """
    Thumbnail filter
    """
    modes = ['inset', 'outbound']
    mode = 'inset'
    width = 0
    height = 0

    def __init__(self, **kwargs):
        """
        Filter initialization
        :param kwargs: parameters
        """
        if 'mode' in kwargs and kwargs['mode'] in self.modes:
            self.mode = kwargs.pop('mode')

        if 'size' in kwargs and isinstance(kwargs['size'], list) and len(kwargs['size']) == 2:
            size = kwargs.pop('size', [0, 0])
            self.width = size[0]
            self.height = size[1]
        else:
            raise ValueError('Thumbnail size is not set.')

    def apply(self, resource):
        """
        Apply filter to resource
        :param resource: Image
        :return: Image
        """
        if not isinstance(resource, Image.Image):
            raise ValueError('Unknown resource format')

        original_width, original_height = resource.size

        if self.mode == 'outbound':
            target_width, target_height = self.outbound_sizes(original_width, original_height, self.width, self.height)
            resource = resource.resize((target_width, target_height), Image.ANTIALIAS)

            crop_sizes = self.crop_sizes(target_width, target_height, self.width, self.height)
            resource = resource.crop(crop_sizes)
        else:
            target_width, target_height = self.inset_sizes(original_width, original_height, self.width, self.height)
            resource = resource.resize((target_width, target_height), Image.ANTIALIAS)

        return resource

    @classmethod
    def inset_sizes(cls, original_width, original_height, target_width, target_height):
        """
        Calculate new image sizes for inset mode
        :param original_width: int
        :param original_height: int
        :param target_width: int
        :param target_height: int
        :return: tuple(int, int)
        """
        if target_width >= original_width and target_height >= original_height:
            target_width = float(original_width)
            target_height = original_height

        elif target_width <= original_width and target_height >= original_height:
            k = original_width / float(target_width)
            target_height = int(original_height / k)

        elif target_width >= original_width and target_height <= original_height:
            k = original_height / float(target_height)
            target_width = int(original_width / k)

        elif target_width < original_width and target_height < original_height:
            k = original_width / float(original_height)
            k_w = original_width / float(target_width)
            k_h = original_height / float(target_height)

            if k_w >= k_h:
                target_height = int(target_width / k)
            else:
                target_width = int(target_height * k)

        return target_width, target_height

    @classmethod
    def outbound_sizes(cls, original_width, original_height, target_width, target_height):
        """
        Calculate new image sizes for outbound mode
        :param original_width: int
        :param original_height: int
        :param target_width: int
        :param target_height: int
        :return: tuple(int, int)
        """
        if target_width <= original_width and target_height <= original_height:
            k = original_width / float(original_height)
            k_w = original_width / float(target_width)
            k_h = original_height / float(target_height)

            if k_w > k_h:
                target_width = int(target_height * k)
            else:
                target_height = int(target_width / k)
        else:
            target_width = original_width
            target_height = original_height

        return target_width, target_height

    @classmethod
    def crop_sizes(cls, original_width, original_height, target_width, target_height):
        """
        Calculate crop parameters for outbound mode
        :param original_width: int
        :param original_height: int
        :param target_width: int
        :param target_height: int
        :return: tuple(int, int, int, int)
        """
        if target_width < original_width:
            left = abs(original_width - target_width) / 2
            right = left + target_width
        else:
            left = 0
            right = original_width
        if target_height < original_height:
            upper = abs(original_height - target_height) / 2
            lower = upper + target_height
        else:
            upper = 0
            lower = original_height

        return left, upper, right, lower



"""
This module implement a Autorotate filter.
"""
from PIL import Image
from .interface import ImagineFilterInterface


class AutorotateFilter(ImagineFilterInterface):
    """
    Autorotate filter
    """
    def apply(self, resource):
        """
        Apply filter to resource
        :param resource: Image
        :return: Image
        """
        if not isinstance(resource, Image.Image):
            raise ValueError('Unknown resource format')

        if hasattr(resource, '_getexif'):
            exif = getattr(resource, '_getexif')()
            if not exif:
                return resource

            orientation_key = 274  # cf ExifTags
            if orientation_key in exif:
                orientation = exif[orientation_key]

                rotate_values = {
                    3: 180,
                    6: 270,
                    8: 90
                }

                if orientation in rotate_values:
                    resource = resource.rotate(rotate_values[orientation])

        return resource



"""
This module implement a Rotate filter.
"""
from PIL import Image
from .interface import ImagineFilterInterface


class RotateFilter(ImagineFilterInterface):
    """
    Rotate filter
    """
    angle = None

    def __init__(self, **kwargs):
        """
        :param kwargs: dict
        """
        if 'angle' in kwargs and isinstance(kwargs['angle'], (int, float)):
            self.angle = kwargs.get('angle', 0)
        else:
            raise ValueError('Unsupported angle format or angle doesn\'t set')

    def apply(self, resource):
        """
        Apply filter to resource
        :param resource: Image
        :return: Image
        """
        if not isinstance(resource, Image.Image):
            raise ValueError('Unknown resource format')

        resource = resource.rotate(self.angle, expand=True)

        return resource



from .autorotate import AutorotateFilter
from .crop import CropFilter
from .downscale import DownscaleFilter
from .interface import ImagineFilterInterface
from .relative_resize import RelativeResizeFilter
from .rotate import RotateFilter
from .thumbnail import ThumbnailFilter
from .upscale import UpscaleFilter
from .watermark import WatermarkFilter

__all__ = [
    'AutorotateFilter',
    'CropFilter',
    'DownscaleFilter',
    'RelativeResizeFilter',
    'RotateFilter',
    'ThumbnailFilter',
    'UpscaleFilter',
    'WatermarkFilter',
    'ImagineFilterInterface'
]



try:
    import multiprocessing
except ImportError:
    pass


from setuptools import setup

setup(
    name='Flask-Imagine',
    version='0.3.1',
    description='Extension which provides easy image manipulation support in Flask applications.',
    url='https://github.com/FlaskGuys/Flask-Imagine',

    author='Kronas',
    author_email='kronas.sw@gmail.com',
    license='MIT',

    packages=['flask_imagine', 'flask_imagine.adapters', 'flask_imagine.filters', 'flask_imagine.helpers'],

    install_requires=[

        'Flask>=0.9',
        'itsdangerous>=0.24',  # For Flask v0.9
        'Pillow==3.1.2',

    ],

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    zip_safe=False,
)



"""
This module implement a filesystem storage adapter.
"""
import errno
import logging
import os
from flask import current_app
from .interface import ImagineAdapterInterface
from PIL import Image

LOGGER = logging.getLogger(__name__)


class ImagineFilesystemAdapter(ImagineAdapterInterface):
    """
    Filesystem storage adapter
    """
    source_folder = None
    cache_folder = None

    def __init__(self, **kwargs):
        """
        Init adapter
        :param kwargs: parameters
        :return:
        """
        if 'source_folder' in kwargs:
            self.source_folder = kwargs.pop('source_folder').strip('/')
        else:
            raise ValueError('Source folder does not set.')

        self.cache_folder = kwargs.pop('cache_folder', 'cache').strip('/')

    def get_item(self, path):
        """
        Get resource item
        :param path: string
        :return: Image
        """
        item_path = '%s/%s/%s' % (
                current_app.root_path,
                self.source_folder,
                path.strip('/')
            )

        if os.path.isfile(item_path):
            try:
                return Image.open(item_path)
            except IOError, e:
                LOGGER.warning('File not found on path "%s" with error: %s' % (item_path, unicode(e)))
                return False
        else:
            return False

    def create_cached_item(self, path, content):
        """
        Create cached resource item
        :param path: string
        :param content: Image
        :return:
        """
        if isinstance(content, Image.Image):
            item_path = '%s/%s/%s/%s' % (
                current_app.root_path,
                self.source_folder,
                self.cache_folder,
                path.strip('/')
            )
            self.make_dirs(item_path)

            content.save(item_path)

            if os.path.isfile(item_path):
                return '/%s/%s/%s' % (self.source_folder, self.cache_folder, path.strip('/'))
            else:  # pragma: no cover
                LOGGER.warning('File is not created on path: %s' % item_path)
                return False
        else:
            return False

    def get_cached_item(self, path):
        """
        Get cached resource item
        :param path: string
        :return:
        """
        item_path = '%s/%s/%s/%s' % (
                current_app.root_path,
                self.source_folder,
                self.cache_folder,
                path.strip('/')
            )

        if os.path.isfile(item_path):
            try:
                return Image.open(item_path)
            except IOError, e:  # pragma: no cover
                LOGGER.warning('Cached file not found on path "%s" with error: %s' % (item_path, unicode(e)))
                return False
        else:
            return False

    def check_cached_item(self, path):
        """
        Check for cached resource item exists
        :param path: string
        :return:
        """
        item_path = '%s/%s/%s/%s' % (
                current_app.root_path,
                self.source_folder,
                self.cache_folder,
                path.strip('/')
            )

        return os.path.isfile(item_path)

    def remove_cached_item(self, path):
        """
        Remove cached resource item
        :param path: string
        :return:
        """
        item_path = '%s/%s/%s/%s' % (
                current_app.root_path,
                self.source_folder,
                self.cache_folder,
                path.strip('/')
            )

        if os.path.isfile(item_path):
            os.remove(item_path)

        return True

    @staticmethod
    def make_dirs(path):
        """
        Create directories if not exist
        :param path: string
        :return:
        """
        try:
            os.makedirs(os.path.dirname(path))
        except OSError as e:
            if e.errno != errno.EEXIST:
                LOGGER.error('Failed to create directory %s with error: %s' % (path, unicode(e)))
                raise



"""
This module implemented a Regexp route handler.
"""
from werkzeug.routing import BaseConverter


class RegexConverter(BaseConverter):
    """
    Regexp url converter.
    """
    def __init__(self, url_map, *items):
        """
        :param url_map:
        :param items:
        :return:
        """
        super(RegexConverter, self).__init__(url_map)
        self.regex = items[0]
